#!BPY

"""
Name: 'ROFF animation (.rof)...'
Blender: 249
Group: 'Export'
Tooltip: 'Save the animation of the selected object as a ROFF'
"""

__author__ = "Willi \"Mr. Wonko\" Schinmeyer"
__url__ = ['www.mrwonko.de']
__version__ = "1.0"

__bpydoc__ = """\
This script is an importer for the ROFF animation file format.

Usage:

Run this script from "File->Import" menu and then select the desired roff file.
The currently selected object will then get the animations from the file applied.
"""

import Blender
import struct
import os

SCALE = 1

def ExportStart(glmfilename):
	objlist = Blender.Object.GetSelected()
	if len(objlist) == 0:
		Blender.Draw.PupBlock("Please select something first.", ["Do it!"])
		return
	obj = objlist[0]
	ipo = obj.getIpo()
	if ipo is None:
		Blender.Draw.PupBlock("The selected object isn't animated.", ["Animate it!"])
		return
	
	okay = False
	locx = ipo[Blender.Ipo.OB_LOCX]
	if locx != None:
		okay = True
	
	locy = ipo[Blender.Ipo.OB_LOCY]
	if locy != None:
		okay = True
	
	locz = ipo[Blender.Ipo.OB_LOCZ]
	if locz != None:
		okay = True
	
	rotx = ipo[Blender.Ipo.OB_ROTX]
	if rotx != None:
		okay = True
	
	roty = ipo[Blender.Ipo.OB_ROTY]
	if roty != None:
		okay = True
	
	rotz = ipo[Blender.Ipo.OB_ROTZ]
	if rotz != None:
		okay = True
	
	if not okay:
		Blender.Draw.PupBlock("Neither location nor rotation are animated.", ["Nothing else is supported."])
		return
	
	with open(glmfilename, "wb") as file:
		context = Blender.Scene.GetCurrent().getRenderingContext()
		frames = context.eFrame - context.sFrame
		# I don't know what the last 8 bytes are, they (nearly) always contain (int) 50 and (int) 0 (once the second contained 7, once 8...)
		file.write(struct.pack("4s4i", b"ROFF", 2, frames, 50, 0))
		
		for frame in range(frames):
			dx = 0
			if locx:
				dx = locx[frame + context.sFrame + 1] - locx[frame + context.sFrame]
			dy = 0
			if locy:
				dy = locy[frame + context.sFrame + 1] - locy[frame + context.sFrame]
			dz = 0
			if locz:
				dz = locz[frame + context.sFrame + 1] - locz[frame + context.sFrame]
			
			drotz = 0
			if rotz:
				drotz = rotz[frame + context.sFrame + 1] - rotz[frame + context.sFrame]
			droty = 0
			if roty:
				droty = roty[frame + context.sFrame + 1] - roty[frame + context.sFrame]
			drotx = 0
			if rotx:
				drotx = rotx[frame + context.sFrame + 1] - rotx[frame + context.sFrame]
			
			# I don't know what the last 8 bytes are, they always contain 0xFFFFFFFF00000000
			file.write(struct.pack("6f2i", dx/SCALE, dy/SCALE, dz/SCALE, droty*10, drotz*10, drotx*10, -1, 0))

def callback_fileselector( filename ):
    if ( os.path.exists( filename ) == 1 ):
        overwrite = Blender.Draw.PupMenu( "File Already Exists, Overwrite?%t|Yes%x1|No%x0" )
        if ( overwrite == 1 ):
            ExportStart( filename )
    else:
        ExportStart( filename )

#open the file selector
Blender.Window.FileSelector(callback_fileselector, 'Export ROFF', 'myanimation.rof')